
#include <proto/exec.h>
#include <proto/dos.h>

#include <exec/interrupts.h>
#include <devices/input.h>
#include <devices/inputevent.h>

#ifdef __PPC__
#define __saveds
#define __A0
#define __A1
#define __A2
#elif defined(__VBCC__)
#define __A0 __reg("a0")
#define __A1 __reg("a1")
#define __A2 __reg("a2")
#endif
#ifndef __amigaos4__
#define NT_EXTINTERRUPT NT_INTERRUPT
#define MEMF_SHARED MEMF_PUBLIC
#endif

__saveds struct InputEvent *handlerproc (__A0 struct InputEvent *ielist,__A1 struct MsgPort *port)

{
struct InputEvent *ie,*prev;

ie = ielist;
prev = NULL;

do	{
	if (ie->ie_Class == IECLASS_RAWMOUSE && ie->ie_Code == IECODE_LBUTTON)
		{
		if (prev)
			prev->ie_NextEvent = ie->ie_NextEvent;
		else
			ielist = ie->ie_NextEvent;
		Signal (port->mp_SigTask,1L << port->mp_SigBit);
		}
	else
		prev = ie;
	ie = ie->ie_NextEvent;
	}
while (ie);

return (ielist);
}


int main (void)
{
int rc = RETURN_FAIL;
struct MsgPort  *port = NULL;
struct Interrupt *handler = NULL;
struct IOStdReq *req = NULL;
BOOL dev_open = FALSE;

if (port = CreateMsgPort())
	if (handler = AllocVec (sizeof(struct Interrupt),MEMF_SHARED|MEMF_CLEAR))
		if (req = (struct IOStdReq *) CreateIORequest (port,sizeof(struct IOStdReq)))
			if (!OpenDevice ("input.device",0,(struct IORequest *)req,0))
				dev_open = TRUE;

if (!dev_open)
	Printf ("could not open input.device\n");
else
	{
	ULONG sigs;

	handler->is_Code = (APTR)handlerproc;
	handler->is_Data = (APTR)port;
	handler->is_Node.ln_Type = NT_EXTINTERRUPT;
	handler->is_Node.ln_Pri  = 60; /* above intuition's handler */
	req->io_Data = (APTR)handler;
	req->io_Command = IND_ADDHANDLER;
	DoIO ((struct IORequest *)req);

/*
   "port" is not used by the device at this time so we can "mis-use" it as
   signal for the mouse click. The MsgPort structure comes handy because it
   already has the task pointer and signal number filled in.
   If we could not use the port, we would have to allocate a second signal and
   put the signal number and a pointer to the main task into is_Data so that
   the handler can use them to signal us.
*/

	SetSignal(0,(1L << port->mp_SigBit)); /* clear port's signal in case DoIO didn't do that */

	sigs = Wait ((1L << port->mp_SigBit) | SIGBREAKF_CTRL_C);

	rc = RETURN_OK;

	if (sigs & SIGBREAKF_CTRL_C)
		{
		Printf ("*** Break\n");
		rc = RETURN_WARN;
		}

	req->io_Data  = (APTR)handler;
	req->io_Command = IND_REMHANDLER;
	DoIO((struct IORequest *)req);
	}

if (dev_open) CloseDevice ((struct IORequest *)req);
if (req)      DeleteIORequest ((struct IORequest *)req);
if (handler)  FreeVec (handler);
if (port)     DeleteMsgPort (port);

return (rc);
}

